package eu.kanade.tachiyomi.ui.reader.setting

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import androidx.core.view.isVisible
import androidx.core.widget.NestedScrollView
import androidx.lifecycle.lifecycleScope
import com.hippo.ehviewer.R
import com.hippo.ehviewer.databinding.ReaderGeneralSettingsBinding
import eu.kanade.tachiyomi.ui.reader.ReaderActivity
import eu.kanade.tachiyomi.util.preference.asHotFlow
import eu.kanade.tachiyomi.util.preference.bindToPreference
import kotlinx.coroutines.flow.launchIn

/**
 * Sheet to show reader and viewer preferences.
 */
class ReaderGeneralSettings @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null) :
    NestedScrollView(context, attrs) {

    private val readerPreferences: ReaderPreferences = ReaderPreferences

    private val binding = ReaderGeneralSettingsBinding.inflate(LayoutInflater.from(context), this, false)

    init {
        addView(binding.root)

        initGeneralPreferences()
    }

    /**
     * Init general reader preferences.
     */
    private fun initGeneralPreferences() {
        binding.backgroundColor.bindToIntPreference(readerPreferences.readerTheme(), R.array.reader_themes_values)
        binding.showPageNumber.bindToPreference(readerPreferences.showPageNumber())
        binding.showReaderSeekbar.bindToPreference(readerPreferences.showReaderSeekbar())
        binding.doubleTapToZoom.bindToPreference(readerPreferences.doubleTapToZoom())
        binding.fullscreen.bindToPreference(readerPreferences.fullscreen())
        readerPreferences.fullscreen()
            .asHotFlow {
                // If the preference is explicitly disabled, that means the setting was configured since there is a cutout
                binding.cutoutShort.isVisible = it && ((context as ReaderActivity).hasCutout || !readerPreferences.cutoutShort().get())
                binding.cutoutShort.bindToPreference(readerPreferences.cutoutShort())
            }
            .launchIn((context as ReaderActivity).lifecycleScope)

        binding.keepscreen.bindToPreference(readerPreferences.keepScreenOn())
        binding.longTap.bindToPreference(readerPreferences.readWithLongTap())
        binding.pageTransitions.bindToPreference(readerPreferences.pageTransitions())
        binding.volumePage.bindToPreference(readerPreferences.readWithVolumeKeys())
        binding.reserveVolumePage.bindToPreference(readerPreferences.readWithVolumeKeysInverted())
        readerPreferences.readWithVolumeKeys().asHotFlow {
            binding.volumePageSpeed.isVisible = it
            binding.reserveVolumePage.isVisible = it
        }.launchIn((context as ReaderActivity).lifecycleScope)

        val volumePageInterval = readerPreferences.readWithVolumeKeysInterval().get()
        binding.sliderVolumePageInterval.value = volumePageInterval.toFloat()
        binding.sliderVolumePageInterval.addOnChangeListener { _, value, fromUser ->
            if (fromUser) {
                readerPreferences.readWithVolumeKeysInterval().set(value.toInt())
            }
        }
    }
}
